# SPDX-FileCopyrightText: 2021-2024 SanderTheDragon <sanderthedragon@zoho.com>
#
# SPDX-License-Identifier: MIT

from functools import partial
from typing import Any

from docutils.parsers.rst.states import Inliner
from docutils.parsers.rst import roles
from docutils import nodes
from sphinx.application import Sphinx
from sphinx.errors import ExtensionError
from sphinx.util.typing import ExtensionMetadata

import sanderthedragon as common


def make_code(name: str, raw: str, text: str, line: int, inliner: Inliner,
              options: dict[str, Any] = {}, content: list[str] = []) \
        -> tuple[list[nodes.Node], list[nodes.system_message]]:
    """
    Very simple wrapper around `code_role` to use 'short' syntax highlighting
    """

    inliner.document.settings.syntax_highlight = 'short'

    return roles.code_role(name, raw, text, line, inliner, options, content)


def setup(app: Sphinx) -> ExtensionMetadata:
    config = app.config._raw_config  # `raw_config` is available in setup

    inline_codes = config.get('inline_codes', {})
    if isinstance(inline_codes, dict):
        for ( key, value ) in inline_codes.items():
            app.add_role(key, partial(make_code, options={
                'language': value, 'classes': [ 'highlight' ]
            }))
    elif isinstance(inline_codes, ( list, tuple, set, frozenset )):
        for key in inline_codes:
            app.add_role(key, partial(make_code, options={
                'language': key, 'classes': [ 'highlight' ]
            }))
    else:
        raise ExtensionError(f'Invalid value type for \'inline_codes\'')

    return { 'version': common.__version__, 'parallel_read_safe': True }



# SPDX-FileCopyrightText: 2021-2024 SanderTheDragon <sanderthedragon@zoho.com>
#
# SPDX-License-Identifier: MIT

from functools import partial
from typing import Any, Optional

from docutils.parsers.rst.states import Inliner
from docutils.nodes import Node
from docutils import nodes
from sphinx.application import Sphinx
from sphinx.util.typing import ExtensionMetadata
from sphinx.util import logging, nodes as nodesutil

import sanderthedragon as common


def make_link(name: str, raw: str, text: str, line: int, inliner: Inliner,
              options: dict[str, Any] = {}, content: list[str] = [],
              mapping: dict[str, str] = {}, target: Optional[str] = None) \
        -> tuple[list[Node], list[nodes.system_message]]:
    # Split text into text, key and anchor
    anchor = None

    ( _, text, key ) = nodesutil.split_explicit_title(text)
    if ':' in key and key != ':':
        ( key, anchor ) = key.rsplit(':', maxsplit=1)

    no_key_anchor = (key == ':')
    if len(key) == 0 or no_key_anchor:
        key = text
        if ':' in key and not no_key_anchor:
            ( key, anchor ) = key.rsplit(':', maxsplit=1)

    # Get the URL
    value = mapping.get(key, None if target is None else key)
    if value is None:
        error = inliner.reporter.error(
            f'"{key}" is not defined for "{name}"', line=line
        )
        problematic = inliner.problematic(rawtext, rawtext, error)

        return ( [ problematic ], [ error ] )

    url = value
    if target is not None:
        url = target.format(value)

    # Append anchor (if given)
    if anchor is not None:
        if not url.startswith('http://') and not url.startswith('https://'):
            anchor = nodes.make_id(anchor)

        url += '#' + anchor

    node = nodes.reference(raw, text, refuri=url)

    return ( [ node ], [] )


def setup(app: Sphinx) -> ExtensionMetadata:
    config = app.config._raw_config  # `raw_config` is available in setup

    role_mapping = config.get('role_mapping', {})
    for ( key, value ) in role_mapping.items():
        item_mapping = config.get(key + '_mapping', {})
        if value is None:
            if len(item_mapping.keys()) == 0:
                logging.getLogger('mappedlinkrole').warning(
                    f'"{key}" is defined as `None`, but no mapping is defined'
                )

            app.add_role(key, partial(make_link, mapping=item_mapping))
        else:
            app.add_role(key, partial(make_link, mapping=item_mapping,
                                      target=value))

    return { 'version': common.__version__, 'parallel_read_safe': True }



# SPDX-FileCopyrightText: 2021-2024 SanderTheDragon <sanderthedragon@zoho.com>
#
# SPDX-License-Identifier: MIT

import json

from pathlib import Path

from docutils import nodes
from pygments.lexers._mapping import LEXERS
from sphinx.application import Sphinx
from sphinx.builders import Builder
from sphinx.util.typing import ExtensionMetadata
from sphinx.util import fileutil

import sanderthedragon as common


def process_languages(app: Sphinx, doctree: nodes.document) -> None:
    """
    Extract the highlight languages from all code blocks in the doctree

    If a language is not yet defined in the 'cb_mimes' option, then it will
    automatically be selected from the pygments lexer mapping
    """

    for block in doctree.traverse(nodes.literal_block):
        if block['language'] not in app.config['cb_mimes']:
            for ( _, ( _, _, language, _, mimes ) ) in LEXERS.items():
                if block['language'] in language and len(mimes) > 0:
                    app.config['cb_mimes'][block['language']] = mimes[0]


def add_js(app: Sphinx, path: Path) -> None:
    source = Path(__file__).parent / '_static' / path
    destination = Path(app.builder.outdir) / '_static' / path

    destination.parent.mkdir(exist_ok=True, parents=True)
    fileutil.copy_asset_file(str(source), str(destination))
    app.add_js_file(str(path))


def add_js_option(app: Sphinx, option: str) -> None:
    # Append it to a file, since adding it inline was adding it twice
    filename = 'cb_options.js'
    path = Path(app.builder.outdir) / '_static' / filename
    with open(path, 'a', encoding='utf-8') as js_file:
        js_file.write(f'const {option} = {json.dumps(app.config[option])};\n')

    app.add_js_file(filename)


def add_css(app: Sphinx, path: Path) -> None:
    source = Path(__file__).parent / '_static' / path
    destination = Path(app.builder.outdir) / '_static' / path

    destination.parent.mkdir(exist_ok=True, parents=True)
    fileutil.copy_asset_file(str(source), str(destination))
    app.add_css_file(str(path))


def add_static(app: Sphinx, builder: Builder) -> None:
    if app.builder.format != 'html':
        return

    add_js(app, Path('clipboard.min.js'))

    add_css(app, Path('codeblock.css'))
    add_js(app, Path('codeblock.js'))

    add_js_option(app, 'cb_default')
    add_js_option(app, 'cb_hidden')
    add_js_option(app, 'cb_icons')
    add_js_option(app, 'cb_mimes')
    add_js_option(app, 'cb_transitions')


def setup(app: Sphinx) -> ExtensionMetadata:
    # The default buttons to show
    app.add_config_value('cb_default', 'cbd-copy', 'html', [ str ])
    # Whether the buttons are hidden by default
    app.add_config_value('cb_hidden', False, 'html', [ bool ])
    # Override icons for the buttons
    app.add_config_value('cb_icons', {}, 'html', [ dict ])
    # The mime types to use for the view button blobs
    app.add_config_value('cb_mimes', {}, 'html', [ dict ])
    # Whether CSS transitions are enabled
    app.add_config_value('cb_transitions', True, 'html', [ bool ])

    app.connect('doctree-read', process_languages)

    app.connect('write-started', add_static)

    return { 'version': common.__version__, 'parallel_read_safe': True }



# SPDX-FileCopyrightText: 2021-2024 SanderTheDragon <sanderthedragon@zoho.com>
#
# SPDX-License-Identifier: MIT

"""
This file just contains the version for Hatch.
It will also be used as extension version.
"""

__version__ = '1.1.0'



# SPDX-FileCopyrightText: 2021-2024 SanderTheDragon <sanderthedragon@zoho.com>
#
# SPDX-License-Identifier: MIT

from pygments.lexer import inherit
from pygments.lexers.javascript import JavascriptLexer
from pygments.token import Comment
from sphinx.application import Sphinx
from sphinx.util.typing import ExtensionMetadata

import sanderthedragon as common


class LLVMJSONLexer(JavascriptLexer):
    """
    The LLVM JSON parser uses '#' instead of '//' for comments.
    This lexer adds it as comment prefix.
    """

    name: str = 'LLVMJSON'
    aliases: list[str] = [ 'llvmjson' ]
    filenames: list[str] = [ '*.imp' ]

    tokens: dict[str, list] = {
        'commentsandwhitespace': [
            ( r'#.*?\n', Comment.Single ),
            inherit
        ]
    }


def setup(app: Sphinx) -> ExtensionMetadata:
    for lexer in [ LLVMJSONLexer ]:
        for alias in [ lexer.name, *lexer.aliases ]:
            app.add_lexer(alias, lexer)

    return { 'version': common.__version__, 'parallel_read_safe': True }
